Package: HiTC
Type: Package
Title: High Throughput Chromosome Conformation Capture analysis
Description: The HiTC package was developed to explore high-throughput 'C' data such as 5C or Hi-C. Dedicated R classes as well as standard methods for quality controls, normalization, visualization, and further analysis are also provided.
Version: 1.56.0
Date: 2025-07-22
Depends: R (>= 2.15.0), methods, IRanges, GenomicRanges
Imports: Biostrings, graphics, grDevices, rtracklayer, RColorBrewer,
        Matrix, parallel, Seqinfo
Suggests: BiocStyle, HiCDataHumanIMR90, BSgenome.Hsapiens.UCSC.hg18
Author: Nicolas Servant
Maintainer: Nicolas Servant <nicolas.servant@curie.fr>
License: Artistic-2.0
Collate: AllGenerics.R HTCexp-class.R HTClist-class.R qualityControl.R
        mapC.R mapC_tracks.R binningC.R export.R import.R pca.R
        normalize_5C.R normalize_hiC.R tads.R deprecated.R
LazyLoad: yes
biocViews: Sequencing, HighThroughputSequencing, HiC
Packaged: 2026-04-28 22:39:16 UTC; biocbuild
InstallableEverywhere: yes
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/HiTC
git_branch: RELEASE_3_23
git_last_commit: 4d550f9
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
