\name{HiTC-internal}
\alias{colorC}
\alias{getDeltaRange}
\alias{tricube}
\alias{gdiag}
\alias{addImageTracks}
\alias{getData2Map}
\alias{heatmapC}
\alias{triViewC}
\alias{pair.chrom}
\alias{my5C2gr}

\title{Internal HiTC functions}
\description{
  Called internally by other \code{HiTC} functions.
  In theory, not called by the user.
}
\author{Nicolas Servant}
\seealso{\code{\link{HTCexp-class}}, \code{\link{HTClist-class}}}
\keyword{internal}
